#define F_CPU 1000000
#include <avr/io.h>
#include <avr/delay.h>
#include <stdlib.h>
#include "lcd-routines.h"
#include "USI_TWI_Master.h"

#define ADS1112_ADDR			0x4E	// (1001110) 4E
#define DS1621_int				0x48	// (1001000) 48
#define DS1621_ext				0x4F	// (1001111) 4F
#define TWI_CMD_MASTER_WRITE	0x10
#define TWI_CMD_MASTER_READ		0x20

#define MESSAGEBUF_SIZE			4



unsigned char messageBuf[MESSAGEBUF_SIZE];
unsigned char TWI_targetSlaveAddress, temp, pressedButton, myCounter=0;



char ausgabe[40];
uint8_t temperatur_msb;
uint8_t temperatur_lsb;

double adc_rechnen(int hb, int lb) {
				
				long t = hb << 8 |  lb;
				if (t >= 32768) t = 65536 *lb - t;				
				double volt = (double) t * 2048/32768;		
				volt = volt*18.55;
				return volt;
}


void ausgabe_display_volt(int zeile, int Messwert) {
	
				
				int vorKomma = Messwert / 1000; // zwei Nachkommastellen "abschneiden"
				int nachKomma = Messwert % 1000; // zwei Nachkommastellen "isolieren"
				lcd_setcursor( 0, zeile );

				utoa( vorKomma, ausgabe,10);
				lcd_string(ausgabe);
				lcd_string(",");
				utoa( nachKomma, ausgabe,10);
				lcd_string(ausgabe);
				lcd_string("  ");
}

void ausgabe_display_temp(int zeile, int hv, int lv) {
	
				lcd_setcursor( 0, zeile );
				
				if (zeile==1)
				{
					lcd_string("A ");
				} else {
					lcd_string("I ");
				}
				utoa( hv, ausgabe,10);
				if (lv == 128) {
					lcd_string(ausgabe);
					lcd_string(",0");
					} else {
					lcd_string(ausgabe);
					lcd_string(",5");
				}
				lcd_string(ausgabe);
				lcd_data(0xDF);
				lcd_string("C  ");
}

int main(void)
{
	
	USI_TWI_Master_Initialise();
	

	lcd_init();
	

	TWI_targetSlaveAddress   = DS1621_int;
	
	messageBuf[0] = (TWI_targetSlaveAddress<<TWI_ADR_BITS) | (FALSE<<TWI_READ_BIT);
	messageBuf[1] = 0xEE;
	temp = USI_TWI_Start_Transceiver_With_Data( messageBuf, 2 );
	while (USI_TWI_Get_State_Info() == USI_TWI_NO_ACK_ON_ADDRESS);

	TWI_targetSlaveAddress   = DS1621_ext;
	
	messageBuf[0] = (TWI_targetSlaveAddress<<TWI_ADR_BITS) | (FALSE<<TWI_READ_BIT);
	messageBuf[1] = 0xEE;
	temp = USI_TWI_Start_Transceiver_With_Data( messageBuf, 2 );
	while (USI_TWI_Get_State_Info() == USI_TWI_NO_ACK_ON_ADDRESS); 
	
	
    while(1)
    {

			TWI_targetSlaveAddress   = DS1621_ext;
			messageBuf[0] = (TWI_targetSlaveAddress<<TWI_ADR_BITS) | (FALSE<<TWI_READ_BIT);
			messageBuf[1] = 0xAA;
			temp = USI_TWI_Start_Transceiver_With_Data( messageBuf, 2 );

		
			messageBuf[0] = (TWI_targetSlaveAddress<<TWI_ADR_BITS) | (TRUE<<TWI_READ_BIT);
			temp = USI_TWI_Start_Transceiver_With_Data( messageBuf, 3 );
			while (USI_TWI_Get_State_Info() == USI_TWI_NO_ACK_ON_ADDRESS);
			
			ausgabe_display_temp(1, messageBuf[1], messageBuf[2]);
			
			TWI_targetSlaveAddress   = DS1621_int;
			messageBuf[0] = (TWI_targetSlaveAddress<<TWI_ADR_BITS) | (FALSE<<TWI_READ_BIT);
			messageBuf[1] = 0xAA;
			temp = USI_TWI_Start_Transceiver_With_Data( messageBuf, 2 );

			
			messageBuf[0] = (TWI_targetSlaveAddress<<TWI_ADR_BITS) | (TRUE<<TWI_READ_BIT);
			temp = USI_TWI_Start_Transceiver_With_Data( messageBuf, 3 );
			while (USI_TWI_Get_State_Info() == USI_TWI_NO_ACK_ON_ADDRESS);
			

			ausgabe_display_temp(2, messageBuf[1], messageBuf[2]);

		
			TWI_targetSlaveAddress   = ADS1112_ADDR;
			messageBuf[0] = (TWI_targetSlaveAddress<<TWI_ADR_BITS) | (FALSE<<TWI_READ_BIT);
			messageBuf[1] = 0xCC;
			temp = USI_TWI_Start_Transceiver_With_Data( messageBuf, 2 );
			while (USI_TWI_Get_State_Info() == USI_TWI_NO_ACK_ON_ADDRESS);
		
	
			messageBuf[0] = (TWI_targetSlaveAddress<<TWI_ADR_BITS) | (TRUE<<TWI_READ_BIT);
			temp = USI_TWI_Start_Transceiver_With_Data( messageBuf, 4 );
			while (USI_TWI_Get_State_Info() == USI_TWI_NO_ACK_ON_ADDRESS);
			_delay_ms(200);
			
			int v = adc_rechnen(messageBuf[1],messageBuf[2]);
			
			ausgabe_display_volt(3,v);

	
			
			TWI_targetSlaveAddress   = ADS1112_ADDR;
			messageBuf[0] = (TWI_targetSlaveAddress<<TWI_ADR_BITS) | (FALSE<<TWI_READ_BIT);
			messageBuf[1] = 0xEC;
			temp = USI_TWI_Start_Transceiver_With_Data( messageBuf, 2 );
			while (USI_TWI_Get_State_Info() == USI_TWI_NO_ACK_ON_ADDRESS);
			
			
			messageBuf[0] = (TWI_targetSlaveAddress<<TWI_ADR_BITS) | (TRUE<<TWI_READ_BIT);
			temp = USI_TWI_Start_Transceiver_With_Data( messageBuf, 4 );
			while (USI_TWI_Get_State_Info() == USI_TWI_NO_ACK_ON_ADDRESS);
			_delay_ms(200);
			
			v = adc_rechnen(messageBuf[1],messageBuf[2]);

			ausgabe_display_volt(4,v);
	

    }
	 return 0;
}